@@ -1,6 +1,6 @@
 /**
  *
- * This file is part of Tulip (http://tulip.labri.fr)
+ * This file is part of Tulip (https://tulip.labri.fr)
  *
  * Authors: David Auber and the Tulip development Team
  * from LaBRI, University of Bordeaux
@@ -19,7 +19,6 @@
 #include <fstream>
 #include <string>
 #include <vector>
-#include <cerrno>
 
 #include <tulip/GraphImpl.h>
 #include <tulip/BooleanProperty.h>
@@ -29,11 +28,8 @@
 #include <tulip/ImportModule.h>
 #include <tulip/IntegerProperty.h>
 #include <tulip/LayoutProperty.h>
-#include <tulip/TulipRelease.h>
-#include <tulip/PropertyTypes.h>
 #include <tulip/SizeProperty.h>
 #include <tulip/StringProperty.h>
-#include <tulip/TlpTools.h>
 #include <tulip/TLPParser.h>
 
 #define TLP "tlp"
@@ -849,11 +845,19 @@ struct TLPDefaultPropertyBuilder : public TLPFalse {
   bool addString(const std::string &val) override {
     if (i == 0) {
       i++;
+      // ensure ascendant compatibility for viewIcon default value
+      // "fas-circle-question" instead of "fa-question-circle" since Tulip 5.7
+      if ((propertyBuilder->property->getName() == "viewIcon") && (val == "fa-question-circle"))
+        return propertyBuilder->setAllNodeValue("fas-circle-question");
       return propertyBuilder->setAllNodeValue(val);
     }
 
     if (i == 1) {
       i++;
+      // ensure ascendant compatibility for viewIcon default value
+      // "fas-circle-question" instead of "fa-question-circle" since Tulip 5.7
+      if ((propertyBuilder->property->getName() == "viewIcon") && (val == "fa-question-circle"))
+        return propertyBuilder->setAllEdgeValue("fas-circle-question");
       return propertyBuilder->setAllEdgeValue(val);
     }
     parser->errorMsg = "invalid property default value format";
@@ -916,21 +920,6 @@ bool TLPGraphBuilder::addStruct(const std::string &structName, TLPBuilder *&newB
 }
 //================================================================================
 
-// Function to retrieve the original size of a file compressed with gzip.
-// The uncompressed file size (modulo 2^32) is stored in the last four bytes of a Gzip file.
-// So that method is unreliable if the original file size was greater than 4GB (which is pretty rare
-// for TLP files ...).
-static int getUncompressedSizeOfGzipFile(const std::string &gzipFilePath) {
-  std::istream *is = tlp::getInputFileStream(gzipFilePath.c_str(), std::ifstream::binary);
-  is->seekg(-4, is->end);
-  int uncompressedSize = 0;
-  is->read(reinterpret_cast<char *>(&uncompressedSize), 4);
-  delete is;
-  return uncompressedSize;
-}
-
-//================================================================================
-
 namespace tlp {
 
 /**
@@ -942,83 +931,89 @@ namespace tlp {
  * choosing "File->Import->TLP" menu item is the same that using
  * "File->Open" menu item.
  */
-class TLPImport : public ImportModule {
+class TLPImport : public ImportFileModule {
+  std::string data;
+
 public:
   PLUGININFORMATION("TLP Import", "Auber", "16/02/2001",
-                    "<p>Supported extensions: tlp, tlpz (compressed), tlp.gz "
+                    "<p>File extensions: tlp, tlpz (compressed), tlp.gz "
                     "(compressed)</p><p>Imports a graph recorded in a file using the TLP format "
                     "(Tulip Software Graph Format).<br/>See "
-                    "<b>http://tulip.labri.fr->Framework->TLP File Format</b> for "
+                    "<a "
+                    "href=\"https://tulip.labri.fr/site/?q=tlp-file-format\">"
+                    "TLP File Format</a> for "
                     "description.<br/>Note: When using the Tulip graphical user "
                     "interface,<br/>choosing <b>File->Import->TLP</b> menu item is the same as "
                     "using <b>File->Open</b> menu item.</p>",
                     "1.0", "File")
-  std::list<std::string> fileExtensions() const override {
-    std::list<std::string> l;
-    l.push_back("tlp");
-    return l;
-  }
 
-  std::list<std::string> gzipFileExtensions() const override {
-    std::list<std::string> ext;
-    ext.push_back("tlp.gz");
-    ext.push_back("tlpz");
-    return ext;
-  }
-
-  TLPImport(tlp::PluginContext *context) : ImportModule(context) {
-    addInParameter<std::string>("file::filename", "The pathname of the TLP file to import.", "");
-  }
+  TLPImport(tlp::PluginContext *context) : ImportFileModule(context, {"tlp", "tlp.gz", "tlpz"}) {}
   ~TLPImport() override {}
 
   std::string icon() const override {
     return ":/tulip/gui/icons/logo32x32.png";
   }
 
-  bool importGraph() override {
-    std::string filename;
-    std::string data;
+  bool check() override {
+    return ImportFileModule::check() ||
+           // file::data is an hidden parameter
+           // used in GraphPerspective.cpp to copy a Graph
+           dataSet->get<std::string>("file::data", data);
+  }
+
+  bool importFile() override {
     std::stringstream *tmpss = nullptr;
     int size;
     std::istream *input;
     bool result;
 
-    if (dataSet->exists("file::filename")) {
-      dataSet->get<std::string>("file::filename", filename);
-      tlp_stat_t infoEntry;
-      result = (statPath(filename, &infoEntry) == 0);
-
-      if (!result) {
-        std::stringstream ess;
-        ess << filename.c_str() << ": " << strerror(errno);
-        pluginProgress->setError(ess.str());
-        tlp::warning() << pluginProgress->getError() << std::endl;
-        return false;
-      }
-
-      size = infoEntry.st_size;
-
-      std::list<std::string> &&gexts = gzipFileExtensions();
+    if (data.empty()) {
       bool gzip(false);
 
-      for (const std::string &ext : gexts) {
-        if (filename.rfind(ext) == (filename.length() - ext.length())) {
-          size = getUncompressedSizeOfGzipFile(filename);
-          input = tlp::getIgzstream(filename);
+      for (auto ext = ++extensions.begin(); ext != extensions.end(); ++ext) {
+        if (filename.rfind(*ext) == (filename.length() - (*ext).length())) {
           gzip = true;
+          // we first open a "standard" stream to retrieve the original size
+          // of the file compressed with gzip.
+          // The uncompressed file size (modulo 2^32) is stored
+          // in the last four bytes of th gzipped file.
+          // This is unreliable if the original file size
+          // was greater than 4GB (which is pretty rare for TLP files ...)
+          input = tlp::getInputFileStream(filename, std::ifstream::binary);
+          if (input->fail())
+            break;
+          input->seekg(-4, input->end);
+          input->read(reinterpret_cast<char *>(&size), 4);
+          delete input;
+
+          input = tlp::getIgzstream(filename);
           break;
         }
       }
 
       if (!gzip)
         input = tlp::getInputFileStream(filename, std::ifstream::in |
-                                                      // consider file has binary
+                                                      // consider file as binary
                                                       // to avoid pb using tellg
                                                       // on the input stream
                                                       std::ifstream::binary);
 
+      // check for open stream failure
+      if (input->fail()) {
+        std::stringstream ess;
+        ess << "Unable to open " << filename << ": " << tlp::getStrError();
+        pluginProgress->setError(ess.str());
+        delete input;
+        return false;
+      }
+
+      if (!gzip) {
+        // get file size
+        input->seekg(0, std::ios::end);
+        size = input->tellg();
+        input->seekg(0, std::ios::beg);
+      }
     } else {
-      dataSet->get<std::string>("file::data", data);
       size = data.size();
       tmpss = new std::stringstream;
       (*tmpss) << data;
@@ -1032,7 +1027,6 @@ public:
 
     if (!result) {
       pluginProgress->setError(filename + ": " + pluginProgress->getError());
-      tlp::warning() << pluginProgress->getError() << std::endl;
     }
 
     delete input;
