@@ -1,6 +1,6 @@
 /**
  *
- * This file is part of Tulip (http://tulip.labri.fr)
+ * This file is part of Tulip (https://tulip.labri.fr)
  *
  * Authors: David Auber and the Tulip development Team
  * from LaBRI, University of Bordeaux
@@ -29,13 +29,14 @@
 #include <tulip/TulipMimes.h>
 #include <tulip/StringProperty.h>
 #include <tulip/TulipMetaTypes.h>
+#include <tulip/TulipSettings.h>
 #include <tulip/TlpQtTools.h>
 #include <tulip/Perspective.h>
 
 #include <QDebug>
 #include <QStandardItemModel>
 #include <QDragEnterEvent>
-#include <QRegExp>
+#include <QRegularExpression>
 
 using namespace tlp;
 using namespace std;
@@ -71,42 +72,20 @@ public:
     return prop;
   }
 };
-class StringSearchOperator : public SearchOperator {
-public:
-  virtual bool compareStrings(const QString &a, const QString &b) = 0;
-  bool compare(tlp::node n) override {
-    return compareStrings(_a->getNodeStringValue(n).c_str(), _b->getNodeStringValue(n).c_str());
-  }
-  bool compare(tlp::edge e) override {
-    return compareStrings(_a->getEdgeStringValue(e).c_str(), _b->getEdgeStringValue(e).c_str());
-  }
-};
-class NumericSearchOperator : public SearchOperator {
-  tlp::NumericProperty *_numericA;
-  tlp::NumericProperty *_numericB;
-
-public:
-  void setProperties(PropertyInterface *a, PropertyInterface *b) override {
-    SearchOperator::setProperties(a, b);
-    _numericA = static_cast<NumericProperty *>(_a);
-    _numericB = static_cast<NumericProperty *>(_b);
-  }
-  virtual bool compareDoubles(double a, double b) = 0;
-  bool compare(tlp::node n) override {
-    return compareDoubles(_numericA->getNodeDoubleValue(n), _numericB->getNodeDoubleValue(n));
-  }
-  bool compare(tlp::edge e) override {
-    return compareDoubles(_numericA->getEdgeDoubleValue(e), _numericB->getEdgeDoubleValue(e));
-  }
-};
 
 #define STRING_CMP(NAME, CMP)                                                                      \
-  class NAME : public StringSearchOperator {                                                       \
+  class NAME : public SearchOperator {                                                             \
   public:                                                                                          \
-    bool compareStrings(const QString &a, const QString &b) {                                      \
+    bool compare(node n) override {                                                                \
+      QString a(_a->getNodeStringValue(n).c_str()), b(_b->getNodeStringValue(n).c_str());          \
+      return CMP;                                                                                  \
+    }                                                                                              \
+    bool compare(edge e) override {                                                                \
+      QString a(_a->getEdgeStringValue(e).c_str()), b(_b->getEdgeStringValue(e).c_str());          \
       return CMP;                                                                                  \
     }                                                                                              \
   };
+
 STRING_CMP(StringEqualsOperator, a == b)
 STRING_CMP(StringDifferentOperator, a != b)
 STRING_CMP(StartsWithOperator, a.startsWith(b))
@@ -118,29 +97,90 @@ STRING_CMP(NoCaseStartsWithOperator, a.startsWith(b, Qt::CaseInsensitive))
 STRING_CMP(NoCaseEndsWithOperator, a.endsWith(b, Qt::CaseInsensitive))
 STRING_CMP(NoCaseContainsOperator, a.contains(b, Qt::CaseInsensitive))
 
-class MatchesOperator : public StringSearchOperator {
+class MatchesOperator : public SearchOperator {
 public:
-  bool compareStrings(const QString &a, const QString &b) override {
-    QRegExp regexp(b);
-    return regexp.exactMatch(a);
+  bool compare(node n) override {
+    QString a(_a->getNodeStringValue(n).c_str());
+    QString b(_b->getNodeStringValue(n).c_str());
+    return QRegularExpression(b).match(a).hasMatch();
+  }
+  bool compare(edge e) override {
+    QString a(_a->getEdgeStringValue(e).c_str());
+    QString b(_b->getEdgeStringValue(e).c_str());
+    return QRegularExpression(b).match(a).hasMatch();
   }
 };
 
-class NoCaseMatchesOperator : public StringSearchOperator {
+class NoCaseMatchesOperator : public SearchOperator {
 public:
-  bool compareStrings(const QString &a, const QString &b) override {
-    QRegExp regexp(b, Qt::CaseInsensitive);
-    return regexp.exactMatch(a);
+  bool compare(node n) override {
+    QString a(_a->getNodeStringValue(n).c_str());
+    QString b(_b->getNodeStringValue(n).c_str());
+    return QRegularExpression(b, QRegularExpression::CaseInsensitiveOption).match(a).hasMatch();
+  }
+  bool compare(edge e) override {
+    QString a(_a->getEdgeStringValue(e).c_str());
+    QString b(_b->getEdgeStringValue(e).c_str());
+    return QRegularExpression(b, QRegularExpression::CaseInsensitiveOption).match(a).hasMatch();
+  }
+};
+
+class IsLikeOperator : public SearchOperator {
+public:
+  bool compare(node n) override {
+    QString a(_a->getNodeStringValue(n).c_str());
+    QString b(_b->getNodeStringValue(n).c_str());
+    convertLikeFilter(b);
+    return QRegularExpression(b).match(a).hasMatch();
+  }
+  bool compare(edge e) override {
+    QString a(_a->getEdgeStringValue(e).c_str());
+    QString b(_b->getEdgeStringValue(e).c_str());
+    convertLikeFilter(b);
+    return QRegularExpression(b).match(a).hasMatch();
+  }
+};
+
+class NoCaseIsLikeOperator : public SearchOperator {
+public:
+  bool compare(node n) override {
+    QString a(_a->getNodeStringValue(n).c_str());
+    QString b(_b->getNodeStringValue(n).c_str());
+    convertLikeFilter(b);
+    return QRegularExpression(b, QRegularExpression::CaseInsensitiveOption).match(a).hasMatch();
+  }
+  bool compare(edge e) override {
+    QString a(_a->getEdgeStringValue(e).c_str());
+    QString b(_b->getEdgeStringValue(e).c_str());
+    convertLikeFilter(b);
+    return QRegularExpression(b, QRegularExpression::CaseInsensitiveOption).match(a).hasMatch();
+  }
+};
+
+class NumericSearchOperator : public SearchOperator {
+protected:
+  tlp::NumericProperty *_numericA;
+  tlp::NumericProperty *_numericB;
+
+public:
+  void setProperties(PropertyInterface *a, PropertyInterface *b) override {
+    SearchOperator::setProperties(a, b);
+    _numericA = static_cast<NumericProperty *>(_a);
+    _numericB = static_cast<NumericProperty *>(_b);
   }
 };
 
 #define NUM_CMP(NAME, CMP)                                                                         \
   class NAME : public NumericSearchOperator {                                                      \
   public:                                                                                          \
-    bool compareDoubles(double a, double b) {                                                      \
-      return a CMP b;                                                                              \
+    bool compare(node n) override {                                                                \
+      return _numericA->getNodeDoubleValue(n) CMP _numericB->getNodeDoubleValue(n);                \
+    }                                                                                              \
+    bool compare(edge e) override {                                                                \
+      return _numericA->getEdgeDoubleValue(e) CMP _numericB->getEdgeDoubleValue(e);                \
     }                                                                                              \
   };
+
 NUM_CMP(DoubleEqualsOperator, ==)
 NUM_CMP(DoubleDifferentOperator, !=)
 NUM_CMP(GreaterOperator, >)
@@ -148,25 +188,74 @@ NUM_CMP(GreaterEqualOperator, >=)
 NUM_CMP(LesserOperator, <)
 NUM_CMP(LesserEqualOperator, <=)
 
+class ConstStringProperty : public StringProperty {
+  std::string _val;
+
+public:
+  ConstStringProperty(Graph *g) : StringProperty(g) {}
+  using StringProperty::operator=;
+  void setConstValue(const std::string &val) {
+    _val = val;
+  }
+
+  std::string getNodeStringValue(const node) const override {
+    return _val;
+  }
+
+  std::string getEdgeStringValue(const edge) const override {
+    return _val;
+  }
+};
+
+class ConstDoubleProperty : public DoubleProperty {
+  double _val;
+
+public:
+  ConstDoubleProperty(Graph *g) : DoubleProperty(g) {}
+  using DoubleProperty::operator=;
+
+  void setConstValue(double val) {
+    _val = val;
+  }
+
+  double getNodeDoubleValue(const node) const override {
+    return _val;
+  }
+
+  double getEdgeDoubleValue(const edge) const override {
+    return _val;
+  }
+};
+
 SearchWidget::SearchWidget(QWidget *parent)
     : QFrame(parent), _ui(new Ui::SearchWidget), _graph(nullptr) {
   _ui->setupUi(this);
+#ifdef __APPLE__
+  _ui->operatorCombo->setMinimumContentsLength(20);
+  _ui->searchTermACombo->setMinimumContentsLength(21);
+  _ui->searchTermBCombo->setMinimumContentsLength(21);
+  _ui->selectionModeCombo->setMinimumContentsLength(30);
+#endif
+  // fix display of QCheckBox and QRadioButton children
+  tlpFixCBRBs(this);
+
   _ui->tableWidget->hide();
   _ui->tableWidget->setItemDelegate(new TulipItemDelegate(_ui->tableWidget));
 
   NUMERIC_OPERATORS << new DoubleEqualsOperator << new DoubleDifferentOperator
                     << new GreaterOperator << new GreaterEqualOperator << new LesserOperator
                     << new LesserEqualOperator << new StartsWithOperator << new EndsWithOperator
-                    << new ContainsOperator << new MatchesOperator;
+                    << new ContainsOperator << new MatchesOperator << new IsLikeOperator;
 
   STRING_OPERATORS << new StringEqualsOperator << new StringDifferentOperator << nullptr << nullptr
                    << nullptr << nullptr << new StartsWithOperator << new EndsWithOperator
-                   << new ContainsOperator << new MatchesOperator;
+                   << new ContainsOperator << new MatchesOperator << new IsLikeOperator;
 
   NOCASE_STRING_OPERATORS << new NoCaseStringEqualsOperator << new NoCaseStringDifferentOperator
                           << nullptr << nullptr << nullptr << nullptr
                           << new NoCaseStartsWithOperator << new NoCaseEndsWithOperator
-                          << new NoCaseContainsOperator << new NoCaseMatchesOperator;
+                          << new NoCaseContainsOperator << new NoCaseMatchesOperator
+                          << new NoCaseIsLikeOperator;
 
   _ui->resultsStorageCombo->setModel(
       new GraphPropertiesModel<BooleanProperty>(nullptr, false, _ui->resultsStorageCombo));
@@ -177,6 +266,7 @@ SearchWidget::SearchWidget(QWidget *parent)
   connect(_ui->graphCombo, SIGNAL(currentItemChanged()), this, SLOT(graphIndexChanged()));
   connect(_ui->selectionModeCombo, SIGNAL(currentIndexChanged(int)), this,
           SLOT(selectionModeChanged(int)));
+  setAcceptDrops(true);
 }
 
 SearchWidget::~SearchWidget() {
@@ -295,12 +385,11 @@ void SearchWidget::search() {
     deleteTermB = true;
 
     if (isNumericComparison()) {
-      DoubleProperty *doubleProp = new DoubleProperty(_graph);
-      doubleProp->setAllNodeValue(_ui->tableWidget->item(0, 0)->data(Qt::DisplayRole).toDouble());
-      doubleProp->setAllEdgeValue(_ui->tableWidget->item(0, 0)->data(Qt::DisplayRole).toDouble());
+      ConstDoubleProperty *doubleProp = new ConstDoubleProperty(_graph);
+      doubleProp->setConstValue(_ui->tableWidget->item(0, 0)->data(Qt::DisplayRole).toDouble());
       b = doubleProp;
     } else {
-      StringProperty *stringProp = new StringProperty(_graph);
+      ConstStringProperty *stringProp = new ConstStringProperty(_graph);
       DataType *tulipData =
           TulipMetaTypes::qVariantToDataType(_ui->tableWidget->item(0, 0)->data(Qt::DisplayRole));
 
@@ -334,8 +423,7 @@ void SearchWidget::search() {
         serializedValue = serializedValue.mid(1, serializedValue.length() - 2);
       }
 
-      stringProp->setAllNodeValue(QStringToTlpString(serializedValue));
-      stringProp->setAllEdgeValue(QStringToTlpString(serializedValue));
+      stringProp->setConstValue(QStringToTlpString(serializedValue));
       b = stringProp;
     }
   } else {
@@ -355,58 +443,51 @@ void SearchWidget::search() {
                                            .value<PropertyInterface *>();
   BooleanProperty *output = static_cast<BooleanProperty *>(outputInterface);
 
-  node n;
-  edge e;
-
   QString searchOpDescription;
-  unsigned int resultsCountNodes = 0, resultsCountEdges = 0;
+  unsigned int resultsCountNodes = result->numberOfNonDefaultValuatedNodes(),
+               resultsCountEdges = result->numberOfNonDefaultValuatedEdges();
 
   if (_ui->selectionModeCombo->currentIndex() == 0) { // replace current selection
     output->copy(result);
     searchOpDescription = "found";
-    resultsCountNodes = iteratorCount(result->getNodesEqualTo(true));
-    resultsCountEdges = iteratorCount(result->getEdgesEqualTo(true));
   } else if (_ui->selectionModeCombo->currentIndex() == 1) { // add to current selection
     if (onNodes) {
-      for (auto n : result->getNodesEqualTo(true)) {
-        output->setNodeValue(n, true);
-        resultsCountNodes++;
+      auto it = result->getNonDefaultValuatedNodes();
+      while (it->hasNext()) {
+        output->setNodeValue(it->next(), true);
       }
+      delete it;
     }
 
     if (onEdges) {
-      for (auto e : result->getEdgesEqualTo(true)) {
-        output->setEdgeValue(e, true);
-        resultsCountEdges++;
+      auto it = result->getNonDefaultValuatedEdges();
+      while (it->hasNext()) {
+        output->setEdgeValue(it->next(), true);
       }
+      delete it;
     }
 
     searchOpDescription = "added to selection";
   } else if (_ui->selectionModeCombo->currentIndex() == 2) { // remove from current selection
     if (onNodes) {
-      for (auto n : output->getNodesEqualTo(true)) {
-        if (result->getNodeValue(n)) {
-          output->setNodeValue(n, false);
-          resultsCountNodes++;
-        }
+      auto it = result->getNonDefaultValuatedNodes();
+      while (it->hasNext()) {
+        output->setNodeValue(it->next(), false);
       }
+      delete it;
     }
 
     if (onEdges) {
-      for (auto e : output->getEdgesEqualTo(true)) {
-        if (result->getEdgeValue(e)) {
-          output->setEdgeValue(e, false);
-          resultsCountEdges++;
-        }
+      auto it = result->getNonDefaultValuatedEdges();
+      while (it->hasNext()) {
+        output->setEdgeValue(it->next(), false);
       }
+      delete it;
     }
 
     searchOpDescription = "removed from selection";
   } else if (_ui->selectionModeCombo->currentIndex() == 3) { // no modification
-    output = result;
     searchOpDescription = "found but not added to selection";
-    resultsCountNodes = iteratorCount(result->getNodesEqualTo(true));
-    resultsCountEdges = iteratorCount(result->getEdgesEqualTo(true));
   }
 
   Observable::unholdObservers();
@@ -503,7 +584,7 @@ void SearchWidget::updateEditorWidget() {
 
   // workaround for sf bug #716
   // I suspect that because 0 == false (when updating termA from viewMetric
-  // from viewSelection) the table item was not properly refreshed.
+  // to viewSelection) the table item was not properly refreshed.
   // So force the refresh of the table item with defaultValue
   // in first displaying a 'blank' value
   _ui->tableWidget->item(0, 0)->setData(Qt::DisplayRole, QVariant(QString()));
@@ -515,15 +596,28 @@ void SearchWidget::dragEnterEvent(QDragEnterEvent *dragEv) {
   const GraphMimeType *mimeType = dynamic_cast<const GraphMimeType *>(dragEv->mimeData());
 
   if (mimeType != nullptr) {
+    QColor color = palette().color(QPalette::Highlight);
+    setStyleSheet(QString("#SearchWidget {background: %0;}").arg(color.name()));
     dragEv->accept();
   }
 }
 
+void SearchWidget::dragLeaveEvent(QDragLeaveEvent *) {
+  if (TulipSettings::isDisplayInDarkMode())
+    setStyleSheet("#SearchWidget {background: #323232;}");
+  else
+    setStyleSheet("#SearchWidget {background: white;}");
+}
+
 void SearchWidget::dropEvent(QDropEvent *dropEv) {
   const GraphMimeType *mimeType = dynamic_cast<const GraphMimeType *>(dropEv->mimeData());
 
   if (mimeType != nullptr) {
     currentGraphChanged(mimeType->graph());
+    if (TulipSettings::isDisplayInDarkMode())
+      setStyleSheet("#SearchWidget {background: #323232;}");
+    else
+      setStyleSheet("#SearchWidget {background: white;}");
     dropEv->accept();
   }
 }
